@@ -18,14 +18,10 @@ class Contour(BaseObject):
 
     **This object posts the following notifications:**
 
-    ===============================
-    Name
-    ===============================
-    Contour.Changed
-    Contour.WindingDirectionChanged
-    Contour.PointsChanged
-    Contour.IdentifierChanged
-    ===============================
+    - Contour.Changed
+    - Contour.WindingDirectionChanged
+    - Contour.PointsChanged
+    - Contour.IdentifierChanged
 
     The Contour object has list like behavior. This behavior allows you to interact
     with point data directly. For example, to get a particular point::
@@ -266,7 +262,7 @@ class Contour(BaseObject):
         """
         Reverse the direction of the contour. It's important to note
         that the actual points stored in this object will be completely
-        repalced by new points.
+        replaced by new points.
 
         This will post *Contour.WindingDirectionChanged*,
         *Contour.PointsChanged* and *Contour.Changed* notifications.
