@@ -1,7 +1,6 @@
 from __future__ import absolute_import
 import weakref
 from warnings import warn
-from fontTools.misc.py23 import basestring
 from fontTools.misc.arrayTools import unionRect
 from defcon.objects.base import BaseObject
 from defcon.objects.contour import Contour
@@ -33,38 +32,42 @@ class Glyph(BaseObject):
 
     **This object posts the following notifications:**
 
-    ============================
-    Name
-    ============================
-    Glyph.Changed
-    Glyph.BeginUndo
-    Glyph.EndUndo
-    Glyph.BeginRedo
-    Glyph.EndRedo
-    Glyph.NameWillChange
-    Glyph.NameChanged
-    Glyph.UnicodesChanged
-    Glyph.WidthChanged
-    Glyph.HeightChanged
-    Glyph.NoteChanged
-    Glyph.LibChanged
-    Glyph.ImageChanged
-    Glyph.ImageWillBeDeleted
-    Glyph.ContourWillBeAdded
-    Glyph.ContourWillBeDeleted
-    Glyph.ContoursChanged
-    Glyph.ComponentWillBeAdded
-    Glyph.ComponentWillBeDeleted
-    Glyph.ComponentsChanged
-    Glyph.AnchorWillBeAdded
-    Glyph.AnchorWillBeDeleted
-    Glyph.AnchorsChanged
-    Glyph.GuidelineWillBeAdded
-    Glyph.GuidelineWillBeDeleted
-    Glyph.GuidelinesChanged
-    Glyph.MarkColorChanged
-    Glyph.VerticalOriginChanged
-    ============================
+    - Glyph.Changed
+    - Glyph.BeginUndo
+    - Glyph.EndUndo
+    - Glyph.BeginRedo
+    - Glyph.EndRedo
+    - Glyph.NameWillChange
+    - Glyph.NameChanged
+    - Glyph.UnicodesChanged
+    - Glyph.WidthChanged
+    - Glyph.HeightChanged
+    - Glyph.LeftMarginWillChange
+    - Glyph.LeftMarginDidChange
+    - Glyph.RightMarginWillChange
+    - Glyph.RightMarginDidChange
+    - Glyph.TopMarginWillChange
+    - Glyph.TopMarginDidChange
+    - Glyph.BottomMarginWillChange
+    - Glyph.BottomMarginDidChange
+    - Glyph.NoteChanged
+    - Glyph.LibChanged
+    - Glyph.ImageChanged
+    - Glyph.ImageWillBeDeleted
+    - Glyph.ContourWillBeAdded
+    - Glyph.ContourWillBeDeleted
+    - Glyph.ContoursChanged
+    - Glyph.ComponentWillBeAdded
+    - Glyph.ComponentWillBeDeleted
+    - Glyph.ComponentsChanged
+    - Glyph.AnchorWillBeAdded
+    - Glyph.AnchorWillBeDeleted
+    - Glyph.AnchorsChanged
+    - Glyph.GuidelineWillBeAdded
+    - Glyph.GuidelineWillBeDeleted
+    - Glyph.GuidelinesChanged
+    - Glyph.MarkColorChanged
+    - Glyph.VerticalOriginChanged
 
     The Glyph object has list like behavior. This behavior allows you to interact
     with contour data directly. For example, to get a particular contour::
@@ -220,7 +223,7 @@ class Glyph(BaseObject):
     def _set_unicodes(self, value):
         oldValue = self.unicodes
         if oldValue != value:
-            self._unicodes = value
+            self._unicodes = list(value)
             self.postNotification(notification="Glyph.UnicodesChanged", data=dict(oldValue=oldValue, newValue=value))
             self.dirty = True
 
@@ -235,11 +238,7 @@ class Glyph(BaseObject):
         if value is None:
             self.unicodes = []
         else:
-            existing = list(self._unicodes)
-            if value in existing:
-                existing.pop(existing.index(value))
-            existing.insert(0, value)
-            self.unicodes = existing
+            self.unicodes = [value]
 
     unicode = property(_get_unicode, _set_unicode, doc="The primary unicode value for the glyph. This is the equivalent of ``glyph.unicodes[0]``. This is a convenience attribute that works with the ``unicodes`` attribute.")
 
@@ -296,11 +295,13 @@ class Glyph(BaseObject):
         oldValue = xMin
         diff = value - xMin
         if value != oldValue:
+            self.postNotification(notification="Glyph.LeftMarginWillChange", data=dict(oldValue=oldValue, newValue=value))
             self.move((diff, 0))
             self.width += diff
             self.dirty = True
+            self.postNotification(notification="Glyph.LeftMarginDidChange", data=dict(oldValue=oldValue, newValue=value))
 
-    leftMargin = property(_get_leftMargin, _set_leftMargin, doc="The left margin of the glyph. Setting this post *Glyph.WidthChanged* and *Glyph.Changed* notifications among others.")
+    leftMargin = property(_get_leftMargin, _set_leftMargin, doc="The left margin of the glyph. Setting this posts *Glyph.WidthChanged*, *Glyph.LeftMarginWillChange*, *Glyph.LeftMarginDidChange* and *Glyph.Changed* notifications among others.")
 
     def _get_rightMargin(self):
         bounds = self.bounds
@@ -316,10 +317,12 @@ class Glyph(BaseObject):
         xMin, yMin, xMax, yMax = bounds
         oldValue = self._width - xMax
         if oldValue != value:
+            self.postNotification(notification="Glyph.RightMarginWillChange", data=dict(oldValue=oldValue, newValue=value))
             self.width = xMax + value
             self.dirty = True
+            self.postNotification(notification="Glyph.RightMarginDidChange", data=dict(oldValue=oldValue, newValue=value))
 
-    rightMargin = property(_get_rightMargin, _set_rightMargin, doc="The right margin of the glyph. Setting this posts *Glyph.WidthChanged* and *Glyph.Changed* notifications among others.")
+    rightMargin = property(_get_rightMargin, _set_rightMargin, doc="The right margin of the glyph. Setting this posts *Glyph.WidthChanged*, *Glyph.RightMarginWillChange*, *Glyph.RightMarginDidChange*  and *Glyph.Changed* notifications among others.")
 
     def _get_bottomMargin(self):
         bounds = self.bounds
@@ -343,10 +346,12 @@ class Glyph(BaseObject):
             oldValue = yMin - (self.verticalOrigin - self.height)
         diff = value - oldValue
         if value != oldValue:
+            self.postNotification(notification="Glyph.BottomMarginWillChange", data=dict(oldValue=oldValue, newValue=value))
             self.height += diff
             self.dirty = True
+            self.postNotification(notification="Glyph.BottomMarginDidChange", data=dict(oldValue=oldValue, newValue=value))
 
-    bottomMargin = property(_get_bottomMargin, _set_bottomMargin, doc="The bottom margin of the glyph. Setting this post *Glyph.HeightChanged* and *Glyph.Changed* notifications among others.")
+    bottomMargin = property(_get_bottomMargin, _set_bottomMargin, doc="The bottom margin of the glyph. Setting this posts *Glyph.HeightChanged*, *Glyph.BottomMarginWillChange*, *Glyph.BottomMarginDidChange*  and *Glyph.Changed* notifications among others.")
 
     def _get_topMargin(self):
         bounds = self.bounds
@@ -369,11 +374,13 @@ class Glyph(BaseObject):
             oldValue = self.verticalOrigin - yMax
         diff = value - oldValue
         if oldValue != value:
+            self.postNotification(notification="Glyph.TopMarginWillChange", data=dict(oldValue=oldValue, newValue=value))
             self.verticalOrigin = yMax + value
             self.height += diff
             self.dirty = True
+            self.postNotification(notification="Glyph.TopMarginWillChange", data=dict(oldValue=oldValue, newValue=value))
 
-    topMargin = property(_get_topMargin, _set_topMargin, doc="The top margin of the glyph. Setting this posts *Glyph.HeightChanged*, *Glyph.VerticalOriginChanged* and *Glyph.Changed* notifications among others.")
+    topMargin = property(_get_topMargin, _set_topMargin, doc="The top margin of the glyph. Setting this posts *Glyph.HeightChanged*, *Glyph.VerticalOriginChanged*, *Glyph.TopMarginWillChange*, *Glyph.TopMarginDidChange*  and *Glyph.Changed* notifications among others.")
 
     # width
 
@@ -1027,7 +1034,7 @@ class Glyph(BaseObject):
 
     def _set_note(self, value):
         if value is not None:
-            assert isinstance(value, basestring)
+            assert isinstance(value, str)
         oldValue = self._note
         if oldValue != value:
             self._note = value
