@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 import weakref
+from fontTools.ufoLib import UFOFileStructure
 from fontTools.misc.arrayTools import unionRect
-from fontTools.misc.py23 import tounicode
 from defcon.objects.base import BaseObject
 from defcon.objects.glyph import Glyph
 from defcon.objects.lib import Lib
@@ -17,31 +17,17 @@ class Layer(BaseObject):
 
     **This object posts the following notifications:**
 
-    +----------------------------+
-    |Name                        |
-    +============================+
-    |Layer.Changed               |
-    +----------------------------+
-    |Layer.GlyphsChanged         |
-    +----------------------------+
-    |Layer.GlyphChanged          |
-    +----------------------------+
-    |Layer.GlyphWillBeAdded      |
-    +----------------------------+
-    |Layer.GlyphAdded            |
-    +----------------------------+
-    |Layer.GlyphWillBeDeleted    |
-    +----------------------------+
-    |Layer.GlyphDeleted          |
-    +----------------------------+
-    |Layer.GlyphNameChanged      |
-    +----------------------------+
-    |Layer.GlyphUnicodesChanged  |
-    +----------------------------+
-    |Layer.NameChanged           |
-    +----------------------------+
-    |Layer.ColorChanged          |
-    +----------------------------+
+    - Layer.Changed
+    - Layer.GlyphsChanged
+    - Layer.GlyphChanged
+    - Layer.GlyphWillBeAdded
+    - Layer.GlyphAdded
+    - Layer.GlyphWillBeDeleted
+    - Layer.GlyphDeleted
+    - Layer.GlyphNameChanged
+    - Layer.GlyphUnicodesChanged
+    - Layer.NameChanged
+    - Layer.ColorChanged
 
     The Layer object has some dict like behavior. For example, to get a glyph::
 
@@ -308,7 +294,7 @@ class Layer(BaseObject):
     # name
 
     def _set_name(self, value):
-        value = tounicode(value)
+        value = str(value)
         oldName = self._name
         if oldName != value:
             self._name = value
@@ -595,10 +581,11 @@ class Layer(BaseObject):
         glyph._dataOnDisk = text
         glyph._dataOnDiskTimeStamp = modTime
 
-    def testForExternalChanges(self):
+    def testForExternalChanges(self, reader):
         """
         Test for external changes. This should not be called externally.
         """
+        self._glyphSet = reader.getGlyphSet(layerName=self.name, validateRead=self.ufoLibReadValidate)
         glyphSet = self._glyphSet
         if glyphSet is None:
             return [], [], []
