@@ -1,6 +1,6 @@
 from __future__ import absolute_import
 import weakref
-from fontTools.ufoLib import UFOReader, UFOFileStructure
+from fontTools.ufoLib import UFOReader, UFOFileStructure, UFOFormatVersion
 from defcon.objects.base import BaseObject
 from defcon.objects.layer import Layer
 
@@ -12,29 +12,16 @@ class LayerSet(BaseObject):
 
     **This object posts the following notifications:**
 
-    +-------------------------------+
-    |Name                           |
-    +===============================+
-    |LayerSet.Changed               |
-    +-------------------------------+
-    |LayerSet.LayersChanged         |
-    +-------------------------------+
-    |LayerSet.LayerChanged          |
-    +-------------------------------+
-    |LayerSet.DefaultLayerWillChange|
-    +-------------------------------+
-    |LayerSet.DefaultLayerChanged   |
-    +-------------------------------+
-    |LayerSet.LayerOrderChanged     |
-    +-------------------------------+
-    |LayerSet.LayerAdded            |
-    +-------------------------------+
-    |LayerSet.LayerDeleted          |
-    +-------------------------------+
-    |LayerSet.LayerWillBeDeleted    |
-    +-------------------------------+
-    |LayerSet.LayerNameChanged      |
-    +-------------------------------+
+    - LayerSet.Changed
+    - LayerSet.LayersChanged
+    - LayerSet.LayerChanged
+    - LayerSet.DefaultLayerWillChange
+    - LayerSet.DefaultLayerChanged
+    - LayerSet.LayerOrderChanged
+    - LayerSet.LayerAdded
+    - LayerSet.LayerDeleted
+    - LayerSet.LayerWillBeDeleted
+    - LayerSet.LayerNameChanged
 
     This object behaves like a dict. For example, to get a particular
     layer::
@@ -302,26 +289,12 @@ class LayerSet(BaseObject):
                     pass
         # save the layers
 
-        # The parent ZipFS is going to be closed inside Font.save, hence
-        # any operations on the GlyphSet's SubFS will fail after that.
-        # To prevent this, we need to reset each layer._glyphSet to None
-        # after saving, when the file structure is zip.
-        # By the time we finish saving a layer, all the glyph data has
-        # been loaded so this is ok. However, setting _glyphSet to None
-        # also means that the Layer.testForExternalChanges method will
-        # produce no effect when the file structure is zip.
-        # This is understandable because only one process at a time
-        # can write to a zip file.
-        isZip = writer.fileStructure is UFOFileStructure.ZIP
-
-        if writer.formatVersion < 3:
+        if writer.formatVersionTuple < UFOFormatVersion.FORMAT_3_0:
             if progressBar is not None:
                 progressBar.update(text="Saving glyphs...", increment=0)
             layer = self.defaultLayer
             glyphSet = writer.getGlyphSet(layerName=None, defaultLayer=True, validateRead=self.ufoLibReadValidate, validateWrite=self.ufoLibWriteValidate)
             layer.save(glyphSet, saveAs=saveAs, progressBar=progressBar)
-            if isZip:
-                layer._glyphSet = None
             if progressBar is not None:
                 progressBar.update()
         else:
@@ -332,8 +305,6 @@ class LayerSet(BaseObject):
                 isDefaultLayer = layer == self.defaultLayer
                 glyphSet = writer.getGlyphSet(layerName=layerName, defaultLayer=isDefaultLayer, validateRead=self.ufoLibReadValidate, validateWrite=self.ufoLibWriteValidate)
                 layer.save(glyphSet, saveAs=saveAs, progressBar=progressBar)
-                if isZip:
-                    layer._glyphSet = None
                 # this prevents us from saving when the color was deleted
                 #if layer.lib or layer.color:
                 glyphSet.writeLayerInfo(layer)
@@ -351,6 +322,22 @@ class LayerSet(BaseObject):
                 continue
             self._layerActionHistory.append(dict(action="new", name=layer.name))
 
+    def _fontSaveWasCompleted(self):
+        """
+        When saving a UFOZ, the underlying ZipFS object is closed.
+        The objects then stored in layer._glyphSet contain references
+        to a closed, and therefore unusable, filesystem. To remedy this,
+        after the save is completed this method will be called and new
+        GlyphSet objects will be created and assigned to the layers.
+        """
+        font = self.font
+        font.close()
+        font._reader = reader = UFOReader(font.path, validate=font.ufoLibReadValidate)
+        if reader.fileStructure is UFOFileStructure.ZIP:
+            for layerName in self.layerOrder:
+                layer = self[layerName]
+                layer._glyphSet = reader.getGlyphSet(layerName=layerName, validateRead=self.ufoLibReadValidate)
+
     # ------------------------
     # Notification Observation
     # ------------------------
@@ -418,13 +405,15 @@ class LayerSet(BaseObject):
         # modified layers
         modifiedLayers = {}
         for layerName in self.layerOrder:
+            if layerName in deletedLayers:
+                continue
             layer = self[layerName]
+            modifiedGlyphs, addedGlyphs, deletedGlyphs = layer.testForExternalChanges(reader)
             newLayerInfo = _StaticLayerInfoMaker()
             layerInfoChanged = False
             if layer._glyphSet is not None:
                 layer._glyphSet.readLayerInfo(newLayerInfo)
                 layerInfoChanged = layer._dataOnDisk != newLayerInfo.pack()
-            modifiedGlyphs, addedGlyphs, deletedGlyphs = layer.testForExternalChanges()
             if modifiedGlyphs or addedGlyphs or deletedGlyphs or layerInfoChanged:
                 modifiedLayers[layerName] = dict(
                     info=layerInfoChanged,
@@ -447,37 +436,37 @@ class LayerSet(BaseObject):
         """
         Reload the layers. This should not be called externally.
         """
-        reader = UFOReader(self.font.path, validate=self.font.ufoLibReadValidate)
-        # handle the layers
-        currentLayerOrder = self.layerOrder
-        for layerName, l in layerData.get("layers", {}).items():
-            # new layer
-            if layerName not in currentLayerOrder:
-                glyphSet = reader.getGlyphSet(layerName, validateRead=self.ufoLibReadValidate, validateWrite=self.font.ufoLibWriteValidate)
-                self.newLayer(layerName, glyphSet)
-            # get the layer
-            layer = self[layerName]
-            # reload the layer info
-            if l.get("info"):
-                layer.color = None
-                layer.lib.clear()
-                layer._glyphSet.readLayerInfo(layer)
-                self._stampLayerInfoDataState(layer)
-            # reload the glyphs
-            glyphNames = l.get("glyphNames", [])
-            if glyphNames:
-                layer.reloadGlyphs(glyphNames)
-        # handle the order
-        if layerData.get("order", False):
-            newLayerOrder = reader.getLayerNames()
-            for layerName in self.layerOrder:
-                if layerName not in newLayerOrder:
-                    newLayerOrder.append(layerName)
-            self.layerOrder = newLayerOrder
-        # handle the default layer
-        if layerData.get("default", False):
-            newDefaultLayerName = reader.getDefaultLayerName()
-            self.defaultLayer = self[newDefaultLayerName]
+        with UFOReader(self.font.path, validate=self.font.ufoLibReadValidate) as reader:
+            # handle the layers
+            currentLayerOrder = self.layerOrder
+            for layerName, l in layerData.get("layers", {}).items():
+                # new layer
+                if layerName not in currentLayerOrder:
+                    glyphSet = reader.getGlyphSet(layerName, validateRead=self.ufoLibReadValidate, validateWrite=self.font.ufoLibWriteValidate)
+                    self.newLayer(layerName, glyphSet)
+                # get the layer
+                layer = self[layerName]
+                # reload the layer info
+                if l.get("info"):
+                    layer.color = None
+                    layer.lib.clear()
+                    layer._glyphSet.readLayerInfo(layer)
+                    self._stampLayerInfoDataState(layer)
+                # reload the glyphs
+                glyphNames = l.get("glyphNames", [])
+                if glyphNames:
+                    layer.reloadGlyphs(glyphNames)
+            # handle the order
+            if layerData.get("order", False):
+                newLayerOrder = reader.getLayerNames()
+                for layerName in self.layerOrder:
+                    if layerName not in newLayerOrder:
+                        newLayerOrder.append(layerName)
+                self.layerOrder = newLayerOrder
+            # handle the default layer
+            if layerData.get("default", False):
+                newDefaultLayerName = reader.getDefaultLayerName()
+                self.defaultLayer = self[newDefaultLayerName]
 
     # -----------------------------
     # Serialization/Deserialization
