@@ -3,14 +3,17 @@ import os
 import glob
 import tempfile
 import shutil
+import fs
+import fs.copy
+import fs.path
 from defcon import Font, Glyph, LayerSet, Guideline
 from defcon.errors import DefconError
 from defcon.tools.notifications import NotificationCenter
 from defcon.test.testTools import (
     getTestFontPath, getTestFontCopyPath, makeTestFontCopy,
+    openTestFontAsFileSystem, closeTestFontAsFileSystem,
     tearDownTestFontCopy)
 from fontTools.ufoLib import UFOReader, UFOWriter, UFOFileStructure
-import fs.path
 import zipfile
 import logging
 from fontTools.misc.loggingTools import CapturingLogHandler
@@ -117,41 +120,53 @@ class FontTest(unittest.TestCase):
             del font["NotInFont"]
 
     def test_delitem_glyph_not_dirty(self):
-        path = makeTestFontCopy()
-        font = Font(path)
-        font["A"]  # glyph = font["A"]
-        glyphPath = os.path.join(path, "glyphs", "A_.glif")
-        os.remove(glyphPath)
-        contentsPath = os.path.join(path, "glyphs", "contents.plist")
-        with open(contentsPath, "rb") as f:
-            plist = load(f)
-        del plist["A"]
-        with open(contentsPath, "wb") as f:
-            dump(plist, f)
-        r = font.testForExternalChanges()
-        self.assertEqual(r["deletedGlyphs"], ["A"])
-        del font["A"]
-        font.save()
-        self.assertFalse(os.path.exists(glyphPath))
+        for ufo in (u"TestExternalEditing.ufo", u"TestExternalEditing.ufoz"):
+            path = getTestFontPath(ufo)
+            path = makeTestFontCopy(path)
+            with Font(path) as font:
+                font["A"]  # glyph = font["A"]
+                fileSystem = openTestFontAsFileSystem(path)
+                glyphPath = fs.path.join("glyphs", "A_.glif")
+                fileSystem.remove(glyphPath)
+                contentsPath = fs.path.join("glyphs", "contents.plist")
+                with fileSystem.open(contentsPath, "rb") as f:
+                    plist = load(f)
+                del plist["A"]
+                with fileSystem.open(contentsPath, "wb") as f:
+                    dump(plist, f)
+                closeTestFontAsFileSystem(fileSystem, path)
+                r = font.testForExternalChanges()
+                self.assertEqual(r["deletedGlyphs"], ["A"])
+                del font["A"]
+                font.save()
+                self.assertFalse(os.path.exists(glyphPath))
+            tearDownTestFontCopy(font.path)
 
     def test_delitem_glyph_dirty(self):
-        path = makeTestFontCopy()
-        font = Font(path)
-        glyph = font["A"]
-        glyph.dirty = True
-        glyphPath = os.path.join(path, "glyphs", "A_.glif")
-        os.remove(glyphPath)
-        contentsPath = os.path.join(path, "glyphs", "contents.plist")
-        with open(contentsPath, "rb") as f:
-            plist = load(f)
-        del plist["A"]
-        with open(contentsPath, "wb") as f:
-            dump(plist, f)
-        r = font.testForExternalChanges()
-        self.assertEqual(r["deletedGlyphs"], ["A"])
-        del font["A"]
-        font.save()
-        self.assertFalse(os.path.exists(glyphPath))
+        for ufo in (u"TestExternalEditing.ufo", u"TestExternalEditing.ufoz"):
+            path = getTestFontPath(ufo)
+            path = makeTestFontCopy(path)
+            with Font(path) as font:
+                glyph = font["A"]
+                glyph.dirty = True
+                fileSystem = openTestFontAsFileSystem(path)
+                glyphPath = fs.path.join("glyphs", "A_.glif")
+                fileSystem.remove(glyphPath)
+                contentsPath = fs.path.join("glyphs", "contents.plist")
+                with fileSystem.open(contentsPath, "rb") as f:
+                    plist = load(f)
+                del plist["A"]
+                with fileSystem.open(contentsPath, "wb") as f:
+                    dump(plist, f)
+                closeTestFontAsFileSystem(fileSystem, path)
+                r = font.testForExternalChanges()
+                self.assertEqual(r["deletedGlyphs"], ["A"])
+                del font["A"]
+                font.save()
+                fileSystem = openTestFontAsFileSystem(path)
+                self.assertFalse(fileSystem.exists(glyphPath))
+                closeTestFontAsFileSystem(fileSystem, path)
+            tearDownTestFontCopy(font.path)
 
     def test_len(self):
         font = Font(getTestFontPath())
@@ -403,27 +418,27 @@ class FontTest(unittest.TestCase):
         self.assertEqual(font.guidelines, [])
 
     def test_save(self):
-        for ufo in ("TestFont.ufo", "TestFont.ufoz"):
+        for ufo in (u"TestFont.ufo", u"TestFont.ufoz"):
             path = makeTestFontCopy(getTestFontPath(ufo))
             try:
-                font = Font(path)
-                origFileStructure = font.ufoFileStructure
-                for glyph in font:
-                    glyph.dirty = True
-                font.save()
-                fileNames = sorted(
-                    [
-                        fs.path.basename(m.path)
-                        for m in UFOReader(path).fs.glob("glyphs/*.glif")
-                    ]
-                )
-                self.assertEqual(fileNames, ["A_.glif", "B_.glif", "C_.glif"])
-                self.assertEqual(origFileStructure, font.ufoFileStructure)
+                with Font(path) as font:
+                    origFileStructure = font.ufoFileStructure
+                    for glyph in font:
+                        glyph.dirty = True
+                    font.save()
+                    fileNames = sorted(
+                        [
+                            fs.path.basename(m.path)
+                            for m in UFOReader(path).fs.glob("glyphs/*.glif")
+                        ]
+                    )
+                    self.assertEqual(fileNames, ["A_.glif", "B_.glif", "C_.glif"])
+                    self.assertEqual(origFileStructure, font.ufoFileStructure)
             finally:
                 tearDownTestFontCopy(path)
 
     def test_save_as(self):
-        for ufo in ("TestFont.ufo", "TestFont.ufoz"):
+        for ufo in (u"TestFont.ufo", u"TestFont.ufoz"):
             path = getTestFontPath(ufo)
             font = Font(path)
             origFileStructure = font.ufoFileStructure
@@ -441,12 +456,14 @@ class FontTest(unittest.TestCase):
                 self.assertEqual(font.path, saveAsPath)
                 self.assertEqual(origFileStructure, font.ufoFileStructure)
             finally:
+                font.close()
                 tearDownTestFontCopy(saveAsPath)
 
     def test_save_same_path(self):
-        for ufo in ("TestFont.ufo", "TestFont.ufoz"):
+        for ufo in (u"TestFont.ufo", u"TestFont.ufoz"):
             path = makeTestFontCopy(getTestFontPath(ufo))
             isZip = zipfile.is_zipfile(path)
+            font = Font(path)
             try:
                 font = Font(path)
                 font.save(path)
@@ -455,18 +472,20 @@ class FontTest(unittest.TestCase):
                 else:
                     self.assertTrue(os.path.isdir(path))
             finally:
+                font.close()
                 tearDownTestFontCopy(path)
 
     def test_save_same_path_different_structure(self):
         for ufo in ("TestFont.ufo", "TestFont.ufoz"):
-            path = getTestFontPath(ufo)
+            path = makeTestFontCopy(getTestFontPath(ufo))
             isZip = zipfile.is_zipfile(path)
-            font = Font(path)
-            with self.assertRaisesRegex(
-                DefconError,
-                "Can't save font in-place with a different structure"
-            ):
-                font.save(path, structure="package" if isZip else "zip")
+            with Font(path) as font:
+                with self.assertRaisesRegex(
+                    DefconError,
+                    "Can't save font in-place with a different structure"
+                ):
+                    font.save(path, structure="package" if isZip else "zip")
+            tearDownTestFontCopy(path)
 
     def test_save_new_font_without_path(self):
         font = Font()
@@ -477,21 +496,22 @@ class FontTest(unittest.TestCase):
     def test_save_new_font_to_exsisting_directory(self):
         for ufo in ("TestFont.ufo", "TestFont.ufoz"):
             path = makeTestFontCopy(getTestFontPath(ufo))
+            font = Font()
             try:
                 self.assertTrue(os.path.exists(path))
-                font = Font()
                 font.save(path)
                 self.assertTrue(os.path.isdir(path))
             finally:
+                font.close()
                 tearDownTestFontCopy(path)
 
     def test_save_ufoz(self):
         path = getTestFontPath()
         tmpdir = tempfile.mkdtemp()
         dest = os.path.join(tmpdir, "TestFont.ufoz")
+        font = Font(path)
         try:
             self.assertFalse(os.path.exists(dest))
-            font = Font(path)
             self.assertEqual(font.path, path)
             font.save(dest, structure="zip")
             self.assertTrue(os.path.exists(dest))
@@ -506,6 +526,7 @@ class FontTest(unittest.TestCase):
             )
             self.assertEqual(fileNames, ["A_.glif", "B_.glif", "C_.glif"])
         finally:
+            font.close()
             shutil.rmtree(tmpdir)
 
     def test_save_new_font_to_existing_file(self):
@@ -523,12 +544,19 @@ class FontTest(unittest.TestCase):
             elif os.path.isdir(path):
                 shutil.rmtree(path)
 
+    def test_new_font_format(self):
+        font = Font()
+        self.assertEqual(font.ufoFormatVersion, None)
+        self.assertEqual(font.ufoFormatVersionTuple, None)
+
     def test_save_in_place_different_format(self):
         path = makeTestFontCopy()
         font = Font(path)
-        self.assertEqual(font._ufoFormatVersion, 3)
+        self.assertEqual(font.ufoFormatVersion, 3)
+        self.assertEqual(font.ufoFormatVersionTuple, (3, 0))
         font.save(formatVersion=2)
-        self.assertEqual(font._ufoFormatVersion, 2)
+        self.assertEqual(font.ufoFormatVersion, 2)
+        self.assertEqual(font.ufoFormatVersionTuple, (2, 0))
 
     def test_save_in_place_invalid_ufo(self):
         path = makeTestFontCopy()
@@ -548,51 +576,51 @@ class FontTest(unittest.TestCase):
         font.save()
 
     def test_testForExternalChanges(self):
-        path = getTestFontPath("TestExternalEditing.ufo")
-        font = Font(path)
+        for ufo in (u"TestExternalEditing.ufo", u"TestExternalEditing.ufoz"):
+            path = getTestFontPath(ufo)
+            path = makeTestFontCopy(path)
+            with Font(path) as font:
+                # load all the objects so that they get stamped
+                font.info  # i = font.info
+                k = font.kerning
+                font.groups  # g = font.groups
+                font.lib  # l = font.lib
+                font["A"]  # g = font["A"]
 
-        # load all the objects so that they get stamped
-        font.info  # i = font.info
-        k = font.kerning
-        font.groups  # g = font.groups
-        font.lib  # l = font.lib
-        font["A"]  # g = font["A"]
+                d = font.testForExternalChanges()
+                self.assertFalse(d["info"])
+                self.assertFalse(d["kerning"])
+                self.assertFalse(d["groups"])
+                self.assertFalse(d["lib"])
 
-        d = font.testForExternalChanges()
-        self.assertFalse(d["info"])
-        self.assertFalse(d["kerning"])
-        self.assertFalse(d["groups"])
-        self.assertFalse(d["lib"])
+                # make a simple change to the kerning data
+                fileSystem = openTestFontAsFileSystem(font.path)
+                path = u"kerning.plist"
+                t = fileSystem.readbytes(path)
+                t += b"<!-- test -->"
+                fileSystem.writebytes(path, t)
+                k._dataOnDiskTimeStamp -= 1
+                closeTestFontAsFileSystem(fileSystem, font.path)
 
-        # make a simple change to the kerning data
-        path = os.path.join(font.path, "kerning.plist")
-        f = open(path, "r")
-        t = f.read()
-        f.close()
-        t += " "
-        f = open(path, "w")
-        f.write(t)
-        f.close()
-        os.utime(path,
-                 (k._dataOnDiskTimeStamp + 1, k._dataOnDiskTimeStamp + 1))
+                d = font.testForExternalChanges()
+                self.assertTrue(d["kerning"])
+                self.assertFalse(d["groups"])
+                self.assertFalse(d["info"])
+                self.assertFalse(d["lib"])
 
-        d = font.testForExternalChanges()
-        self.assertTrue(d["kerning"])
-        self.assertFalse(d["groups"])
-        self.assertFalse(d["info"])
-        self.assertFalse(d["lib"])
+                # save the kerning data and test again
+                font.kerning.dirty = True
+                font.save()
+                d = font.testForExternalChanges()
+                self.assertFalse(d["kerning"])
+                self.assertFalse(d["groups"])
+                self.assertFalse(d["info"])
+                self.assertFalse(d["lib"])
 
-        # save the kerning data and test again
-        font.kerning.dirty = True
-        font.save()
-        d = font.testForExternalChanges()
-        self.assertFalse(d["kerning"])
-        self.assertFalse(d["groups"])
-        self.assertFalse(d["info"])
-        self.assertFalse(d["lib"])
+            tearDownTestFontCopy(font.path)
 
     def test_reloadInfo(self):
-        path = getTestFontPath("TestExternalEditing.ufo")
+        path = getTestFontPath(u"TestExternalEditing.ufo")
         font = Font(path)
         info = font.info
 
@@ -615,7 +643,7 @@ class FontTest(unittest.TestCase):
         f.close()
 
     def test_reloadKerning(self):
-        path = getTestFontPath("TestExternalEditing.ufo")
+        path = getTestFontPath(u"TestExternalEditing.ufo")
         font = Font(path)
         kerning = font.kerning
 
@@ -638,7 +666,7 @@ class FontTest(unittest.TestCase):
         f.close()
 
     def test_reloadGroups(self):
-        path = getTestFontPath("TestExternalEditing.ufo")
+        path = getTestFontPath(u"TestExternalEditing.ufo")
         font = Font(path)
         groups = font.groups
 
@@ -661,7 +689,7 @@ class FontTest(unittest.TestCase):
         f.close()
 
     def test_reloadLib(self):
-        path = getTestFontPath("TestExternalEditing.ufo")
+        path = getTestFontPath(u"TestExternalEditing.ufo")
         font = Font(path)
         lib = font.lib
 
@@ -686,7 +714,7 @@ class FontTest(unittest.TestCase):
         f.close()
 
     def test_reloadGlyphs(self):
-        path = getTestFontPath("TestExternalEditing.ufo")
+        path = getTestFontPath(u"TestExternalEditing.ufo")
         font = Font(path)
         glyph = font["A"]
 
